var client = require("./client");
var commands = require("./commands");

// https://jlptsensei.com/jlpt-n5-vocabulary-list/
var jlptn5words = [["1","浴びる","abiru\n\nあびる","Verb, Ichidan verb, Transitive verb","to bathe, to shower"],["2","危ない","abunai\n\nあぶない","Adjective, い-adjective","dangerous"],["3","あっち","acchi","Pronoun","over there"],["4","あちら","achira","Pronoun","there"],["5","上げる","ageru\n\nあげる","Verb, Ichidan verb, Transitive verb","to raise; to elevate"],["6","赤","aka\n\nあか","Noun","red; crimson; scarlet​"],["7","赤い","akai\n\nあかい","Adjective, い-adjective","red; crimson; scarlet​"],["8","明るい","akarui\n\nあかるい","Adjective, い-adjective","bright; light"],["9","開ける","akeru\n\nあける","Verb, Ichidan verb, Transitive verb","to open (a door, etc.); to unwrap (e.g. parcel, package); to unlock"],["10","秋","aki\n\nあき","Noun","autumn; fall"],["11","開く","aku\n\nあく","Verb, Godan verb, Intransitive verb","to open (e.g. doors, business, etc)"],["12","甘い","amai\n\nあまい","Adjective, い-adjective","sweet; sweet-tasting; sugary; naive; indulgent"],["13","飴","ame\n\nあめ","Noun","candy"],["14","雨","ame\n\nあめ","Noun","rain"],["15","あなた","anata","Pronoun","you"],["16","姉","ane\n\nあね","Noun","older sister; elder sister​"],["17","兄","ani\n\nあに","Noun","elder brother; older brother​"],["18","あの","ano","Pre-noun adjectival","that"],["19","青","ao\n\nあお","Noun","blue; azure"],["20","青い","aoi\n\nあおい","Adjective, い-adjective","blue; azure"],["21","アパート","apaato","Noun, Katakana","apartment"],["22","洗う","arau\n\nあらう","Verb, Godan verb, Transitive verb","to wash"],["23","あれ","are","Pronoun","that"],["24","ある","aru","Verb, Godan verb, Intransitive verb","to be, to have"],["25","歩く","aruku\n\nあるく","Verb, Godan verb, Intransitive verb","to walk"],["26","朝","asa\n\nあさ","Noun","morning"],["27","朝ご飯","asagohan\n\nあさごはん","Noun","breakfast"],["28","明後日","asatte\n\nあさって","Noun","day after tomorrow"],["29","足","ashi\n\nあし","Noun","foot; leg; paw; arm"],["30","明日","ashita\n\nあした","Noun, Temporal noun","tomorrow"],["31","遊ぶ","asobu\n\nあそぶ","Verb, Godan verb, Intransitive verb","to play; to enjoy oneself"],["32","あそこ","asoko","Pronoun","over there"],["33","頭","atama\n\nあたま","Noun","head"],["34","新しい","atarashii\n\nあたらしい","Adjective, い-adjective","new; novel; fresh; recent; latest"],["35","暖かい","atatakai\n\nあたたかい","Adjective, い-adjective","warm"],["36","後","ato\n\nあと","Noun","behind; after; remainder; left; also"],["37","厚い","atsui\n\nあつい","Adjective, い-adjective","thick"],["38","暑い","atsui\n\nあつい","Adjective, い-adjective","hot; sultry"],["39","熱い","atsui\n\nあつい","Adjective, い-adjective","hot"],["40","会う","au\n\nあう","Verb, Godan verb, Intransitive verb","to meet; to encounter; to see"],["41","晩ご飯","bangohan\n\nばんごはん","Noun","dinner; evening meal"],["42","番号","bangou\n\nばんごう","Noun","number"],["43","バス","basu","Noun, Katakana","bus"],["44","バター","bataa","Noun, Katakana","butter​"],["45","ベッド","beddo","Noun, Katakana","bed"],["46","勉強","benkyou\n\nべんきょう","Noun, Suru verb","to study"],["47","便利","benri\n\nべんり","な-adjective","convenient; handy; useful"],["48","ボールペン","boorupen","Noun, Katakana","ball-point pen"],["49","ボタン","botan","Noun, Katakana","button"],["50","帽子","boushi\n\nぼうし","Noun","hat; cap"],["51","文章","bunshou\n\nぶんしょう","Noun","sentence"],["52","豚肉","butaniku\n\nぶたにく","Noun","pork"],["53","病院","byouin\n\nびょういん","Noun","hospital"],["54","病気","byouki\n\nびょうき","Noun","illness; disease; sickness"],["55","茶色","chairo\n\nちゃいろ","Noun","brown"],["56","茶碗","chawan\n\nちゃわん","Noun","rice bowl; tea cup; teacup"],["57","父","chichi\n\nちち","Noun","father"],["58","違う","chigau\n\nちがう","Verb, Godan verb, Intransitive verb","to differ"],["59","小さい","chiisai\n\nちいさい","Adjective, い-adjective","small; little; tiny"],["60","小さな","chiisana\n\nちいさな","Pre-noun adjectival","small; little; tiny​"],["61","近い","chikai\n\nちかい","い-adjective","near; close"],["62","地下鉄","chikatetsu\n\nちかてつ","Noun","subway; underground train"],["63","地図","chizu\n\nちず","Noun","map"],["64","ちょっと","chotto","Adverb","a little"],["65","丁度","choudo\n\nちょうど","Adverb","exactly"],["66","台所","daidokoro\n\nだいどころ","Noun","kitchen"],["67","大学","daigaku\n\nだいがく","Noun","university; college"],["68","大丈夫","daijoubu\n\nだいじょうぶ","Adjective, な-adjective, Adverb","OK; okay; alright; problem free"],["69","大好き","daisuki\n\nだいすき","Adjective, な-adjective","love; like; like very much"],["70","だんだん","dandan","Adverb","gradually"],["71","誰","dare\n\nだれ","Pronoun","who"],["72","誰か","dareka\n\nだれか","Pronoun","someone; somebody"],["73","出す","dasu\n\nだす","Verb, Godan verb, Transitive verb","to take out; to get out; to put out; to reveal"],["74","出口","deguchi\n\nでぐち","Noun","exit; gateway; way out"],["75","出かける","dekakeru\n\nでかける","Verb, Ichidan verb, Intransitive verb","to go out; to leave; to depart"],["76","電気","denki\n\nでんき","Noun","electricity"],["77","電車","densha\n\nでんしゃ","Noun","train; electric train"],["78","電話","denwa\n\nでんわ","Noun, Verb, Suru verb","telephone (call / device)l; phone call"],["79","デパート","depaato","Noun, Katakana","department store"],["80","出る","deru\n\nでる","Verb, Ichidan verb, Intransitive verb","to leave; to exit; to appear; to go out"],["81","ドア","doa","Noun, Katakana","door"],["82","どっち","docchi","Pronoun","which; which one"],["83","どちら","dochira","Pronoun","which of two"],["84","どこ","doko","Pronoun","where; what place​"],["85","どなた","donata","Noun","who"],["86","どの","dono","Pre-noun adjectival","which"],["87","どれ","dore","Pronoun","which (of three or more)​"],["88","どう","dou","Adverb","how; in what way; how about​"],["89","動物","doubutsu\n\nどうぶつ","Noun","animal"],["90","どうも","doumo","Adverb","thank you; thanks"],["91","どうぞ","douzo","Adverb","please"],["92","土曜日","doyoubi\n\nどようび","Noun","Saturday"],["93","絵","e\n\nえ","Noun","picture"],["94","ええ","ee","Noun","yes; that is correct; right"],["95","映画","eiga\n\nえいが","Noun","movie; film"],["96","映画館","eigakan\n\nえいがかん","Noun","movie theater; cinema"],["97","英語","eigo\n\nえいご","Noun","English language"],["98","駅","eki\n\nえき","Noun","station"],["99","鉛筆","enpitsu\n\nえんぴつ","Noun","pencil"],["100","エレベーター","erebeetaa","Noun, Katakana","elevator"],["101","フィルム","firumu","Noun, Katakana","film"],["102","フォーク","fooku","Noun, Katakana","fork"],["103","吹く","fuku\n\nふく","Verb, Godan verb, Intransitive verb","to blow (of the wind)"],["104","服","fuku\n\nふく","Noun","clothes"],["105","降る","furu\n\nふる","Verb, Godan verb, Intransitive verb","to fall"],["106","古い","furui\n\nふるい","い-adjective","old (not used for people)"],["107","二人","futari\n\nふたり","Noun","two people; pair; couple"],["108","二つ","futatsu\n\nふたつ","Noun, Numeric","two; 2"],["109","太い","futoi\n\nふとい","Adjective, い-adjective","fat; thick"],["110","二日","futsuka\n\nふつか","Noun","the second day of the month / 2 days"],["111","封筒","fuutou\n\nふうとう","Noun","envelope"],["112","冬","fuyu\n\nふゆ","Noun","winter"],["113","外国","gaikoku\n\nがいこく","Noun","foreign country"],["114","外国人","gaikokujin\n\nがいこくじん","Noun","foreigner; foreign citizen; foreign national; alien; non-Japanese"],["115","学校","gakkou\n\nがっこう","Noun","school"],["116","学生","gakusei\n\nがくせい","Noun","student"],["117","玄関","genkan\n\nげんかん","Noun","entrance"],["118","元気","genki\n\nげんき","Noun, Adjective, な-adjective","lively; full of spirit; energetic; healthy"],["119","月曜日","getsuyoubi\n\nげつようび","Noun","Monday"],["120","銀行","ginkou\n\nぎんこう","Noun","bank"],["121","ギター","gitaa","Noun, Katakana","guitar"],["122","五","go\n\nご","Noun, Numeric","five; 5"],["123","午後","gogo\n\nごご","Noun","afternoon; p.m."],["124","ご飯","gohan\n\nごはん","Noun","cooked rice, meal"],["125","午前","gozen\n\nごぜん","Noun","morning; a.m."],["126","グラム","guramu","Noun, Katakana","gram"],["127","牛肉","gyuuniku\n\nぎゅうにく","Noun","beef"],["128","牛乳","gyuunyuu\n\nぎゅうにゅう","Noun","(cow's) milk"],["129","歯","ha\n\nは","Noun","tooth"],["130","八","hachi\n\nはち","Noun, Numeric","eight: 8"],["131","葉書","hagaki\n\nはがき","Noun","postcard"],["132","母","haha\n\nはは","Noun","mother"],["133","はい","hai","Noun","yes; that is correct​"],["134","入る","hairu\n\nはいる","Verb, Godan verb, Intransitive verb","to enter; to go into"],["135","灰皿","haizara\n\nはいざら","Noun","ashtray"],["136","始まる","hajimaru\n\nはじまる","Verb, Godan verb, Intransitive verb","to begin"],["137","初めて","hajimete\n\nはじめて","Adverb","for the first time"],["138","箱","hako\n\nはこ","Noun","box; crate"],["139","履く","haku\n\nはく","Verb, Godan verb, Transitive verb","to wear, to put on trousers"],["140","半","han\n\nはん","Noun","half; semi-; half-past"],["141","花","hana\n\nはな","Noun","flower"],["142","鼻","hana\n\nはな","Noun","nose"],["143","話","hanashi\n\nはなし","Noun","talk; speech; chat; conversation​"],["144","話す","hanasu\n\nはなす","Verb, Godan verb, Transitive verb","to speak; to talk; to converse"],["145","半分","hanbun\n\nはんぶん","Noun","half"],["146","ハンカチ","hankachi","Noun, Katakana","handkerchief​"],["147","晴れ","hare\n\nはれ","Noun","clear weather"],["148","晴れる","hareru\n\nはれる","Verb, Ichidan verb, Intransitive verb","to be sunny"],["149","貼る","haru\n\nはる","Verb, Godan verb, Transitive verb","to stick; to paste"],["150","春","haru\n\nはる","Noun","spring; springtime"],["151","箸","hashi\n\nはし","Noun","chopsticks"],["152","橋","hashi\n\nはし","Noun","bridge"],["153","走る","hashiru\n\nはしる","Verb, Godan verb, Intransitive verb","to run"],["154","二十歳","hatachi\n\nはたち","Noun","20 years old; twenty years old"],["155","働く","hataraku\n\nはたらく","Verb, Godan verb, Intransitive verb","to work"],["156","二十日","hatsuka\n\nはつか","Noun","twentieth day of the month / 20 days"],["157","早い","hayai\n\nはやい","Adjective, い-adjective","fast; early"],["158","速い","hayai\n\nはやい","Adjective, い-adjective","fast; quick; hasty; brisk"],["159","辺","hen\n\nへん","Noun","area"],["160","下手","heta\n\nへた","Noun, Adjective, な-adjective","unskillful; poor; awkward​"],["161","部屋","heya\n\nへや","Noun","room"],["162","左","hidari\n\nひだり","Noun","left; left hand side"],["163","東","higashi\n\nひがし","Noun","east"],["164","飛行機","hikouki\n\nひこうき","Noun","airplane; aircraft"],["165","引く","hiku\n\nひく","Verb, Godan verb, Transitive verb","to pull"],["166","弾く","hiku\n\nひく","Verb, Godan verb, Transitive verb","to play"],["167","低い","hikui\n\nひくい","い-adjective","short,low"],["168","暇","hima\n\nひま","Noun, な-adjective","free time"],["169","広い","hiroi\n\nひろい","い-adjective","spacious; vast; wide"],["170","昼","hiru\n\nひる","Noun","noon; midday; daytime; lunch"],["171","昼ご飯","hirugohan\n\nひるごはん","Noun","lunch"],["172","人","hito\n\nひと","Noun","person; human"],["173","一人","hitori\n\nひとり","Noun, Adverb","one person​; alone; single"],["174","一つ","hitotsu\n\nひとつ","Noun, Numeric","one thing; only"],["175","ほか","hoka","Noun","other (place, thing, person); the rest"],["176","本","hon\n\nほん","Noun","book; volume; script"],["177","本棚","hondana\n\nほんだな","Noun","bookshelf; bookcase"],["178","本当","hontou\n\nほんとう","Noun, Adjective, な-adjective","truth; reality; actuality; fact"],["179","欲しい","hoshii\n\nほしい","Adjective, い-adjective","want"],["180","細い","hosoi\n\nほそい","Adjective, い-adjective","thin; slender"],["181","ホテル","hoteru","Noun, Katakana","hotel"],["182","百","hyaku\n\nひゃく","Noun, Numeric","100; hundred"],["183","一","ichi\n\nいち","Noun, Prefix, Suffix, Numeric","one; best; first; foremost; start"],["184","一番","ichiban\n\nいちばん","Noun, Adverb","number one; first; 1st, first place​; best; most​"],["185","一日","ichinichi\n\nいちにち","Noun, Adverbial Noun","one day, all day"],["186","家","ie\n\nいえ","Noun","house, residence, family"],["187","いかが","ikaga","Adverb","how; in what way; how about​"],["188","池","ike\n\nいけ","Noun","pond"],["189","行く","iku\n\nいく","Verb, Godan verb, Intransitive verb","to go; to move"],["190","いくら","ikura","Noun, Adverb","how much?; how many?​"],["191","いくつ","ikutsu","Adverb","how many?,how old?"],["192","今","ima\n\nいま","Noun, Adverbial Noun","now; the present time; soon"],["193","意味","imi\n\nいみ","Noun","meaning; significance; sense"],["194","妹","imouto\n\nいもうと","Noun","younger sister"],["195","犬","inu\n\nいぬ","Noun","dog"],["196","入れる","ireru\n\nいれる","Verb, Ichidan verb, Transitive verb","to put in; to let in; to take in; to bring in; to insert; to install"],["197","入口","iriguchi\n\nいりぐち","Noun","entrance; entry; gate"],["198","色","iro\n\nいろ","Noun","colour; color"],["199","色々","iroiro\n\nいろいろ","Noun, Adjective, な-adjective, Adverb","various"],["200","居る","iru\n\nいる","Verb, Ichidan verb, Intransitive verb","to be, to have"],["201","要る","iru\n\nいる","Verb, Godan verb, Intransitive verb","to be needed"],["202","医者","isha\n\nいしゃ","Noun","(medical) doctor; physician"],["203","忙しい","isogashii\n\nいそがしい","Adjective, い-adjective","busy"],["204","一緒","issho\n\nいっしょ","Noun","together; at the same time; same; identical"],["205","椅子","isu\n\nいす","Noun","chair"],["206","痛い","itai\n\nいたい","Adjective, い-adjective","painful; sore​"],["207","いつ","itsu","Pronoun","when"],["208","五日","itsuka\n\nいつか","Noun","the fifth day of the month / 5 days"],["209","五つ","itsutsu\n\nいつつ","Noun, Numeric","five; 5"],["210","言う","iu\n\nいう","Verb, Godan verb","to say; to call"],["211","嫌","iya\n\nいや","Noun, Adjective, な-adjective","unpleasant"],["212","じゃあ","jaa","Conjunction","then; well; so; well then"],["213","字引","jibiki\n\nじびき","Noun","dictionary"],["214","自分","jibun\n\nじぶん","Pronoun","myself; yourself; oneself; himself; herself; i; me"],["215","自動車","jidousha\n\nじどうしゃ","Noun","automobile; motorcar; motor vehicle; car"],["216","時間","jikan\n\nじかん","Noun","time; hour(s)"],["217","辞書","jisho\n\nじしょ","Noun","dictionary"],["218","自転車","jitensha\n\nじてんしゃ","Noun","bicycle"],["219","丈夫","joubu\n\nじょうぶ","Adjective, な-adjective","strong, durable"],["220","上手","jouzu\n\nじょうず","Noun, Adjective, な-adjective","skillful; skilled; proficient; good (at)"],["221","授業","jugyou\n\nじゅぎょう","Noun, Suru verb","lesson; class work"],["222","十","juu\n\nじゅう","Noun, Numeric","ten; 10"],["223","かばん","kaban","Noun","bag; basket​"],["224","花瓶","kabin\n\nかびん","Noun","a vase"],["225","角","kado\n\nかど","Noun","a corner; angle​"],["226","帰る","kaeru\n\nかえる","Verb, Godan verb, Intransitive verb","to go back​"],["227","返す","kaesu\n\nかえす","Verb, Godan verb, Transitive verb","to return something"],["228","鍵","kagi\n\nかぎ","Noun","key"],["229","階段","kaidan\n\nかいだん","Noun","stairs; stairway; staircase"],["230","買い物","kaimono\n\nかいもの","Noun","shopping; purchased goods"],["231","会社","kaisha\n\nかいしゃ","Noun","company; corporation"],["232","掛かる","kakaru\n\nかかる","Verb, Godan verb, Intransitive verb","to take (a resource, e.g. time or money)"],["233","掛ける","kakeru\n\nかける","Verb, Ichidan verb, Transitive verb","to hang up; to make (a call)​;"],["234","書く","kaku\n\nかく","Verb, Godan verb, Transitive verb","to write; to compose; to pen; to draw"],["235","カメラ","kamera","Noun, Katakana","camera"],["236","紙","kami\n\nかみ","Noun","paper"],["237","漢字","kanji\n\nかんじ","Noun","kanji"],["238","カップ","kappu","Noun, Katakana","cup"],["239","体","karada\n\nからだ","Noun","body"],["240","辛い","karai\n\nからい","い-adjective","spicy"],["241","カレー","karee","Noun, Katakana","curry"],["242","カレンダー","karendaa","Noun, Katakana","calendar"],["243","借りる","kariru\n\nかりる","Verb, Ichidan verb, Transitive verb","to borrow"],["244","軽い","karui\n\nかるい","Adjective, い-adjective","light"],["245","傘","kasa\n\nかさ","Noun","umbrella"],["246","貸す","kasu\n\nかす","Verb, Godan verb, Transitive verb","to lend; to loan"],["247","方","kata\n\nかた","Noun","way of doing something"],["248","家庭","katei\n\nかてい","Noun","household"],["249","買う","kau\n\nかう","Verb, Godan verb, Transitive verb","to buy; to purchase"],["250","川","kawa\n\nかわ","Noun, Suffix","river; stream​"],["251","可愛い","kawaii\n\nかわいい","Adjective, い-adjective","cute"],["252","火曜日","kayoubi\n\nかようび","Noun","Tuesday"],["253","風邪","kaze\n\nかぜ","Noun","a cold"],["254","風","kaze\n\nかぜ","Noun","wind"],["255","家族","kazoku\n\nかぞく","Noun","family; members of a family"],["256","警官","keikan\n\nけいかん","Noun","policeman; police officer"],["257","結婚","kekkon\n\nけっこん","Noun, Suru verb","marriage"],["258","結構","kekkou\n\nけっこう","Noun, Adverbial Noun, Adjective, な-adjective, Adverb","splendid, enough"],["259","今朝","kesa\n\nけさ","Noun, Temporal noun","this morning"],["260","消す","kesu\n\nけす","Verb, Godan verb, Transitive verb","to erase, to turn off power"],["261","木","ki\n\nき","Noun","tree; shrub; bush; wood; timber"],["262","消える","kieru\n\nきえる","Verb, Ichidan verb, Intransitive verb","to disappear"],["263","黄色い","kiiroi\n\nきいろい","Adjective, い-adjective","yellow"],["264","聞く","kiku\n\nきく","Verb, Godan verb, Transitive verb","to hear; to listen (to music); to ask; to learn of"],["265","昨日","kinou\n\nきのう","Noun","yesterday"],["266","金曜日","kinyoubi\n\nきんようび","Noun","Friday"],["267","切符","kippu\n\nきっぷ","Noun","ticket"],["268","嫌い","kirai\n\nきらい","Noun, Adjective, な-adjective","hate"],["269","綺麗","kirei\n\nきれい","Adjective, な-adjective","pretty; lovely; beautiful"],["270","キログラム","kiro guramu","Noun, Katakana","kilogram"],["271","キロメートル","kiro meetoru","Noun, Katakana","kilometer"],["272","切る","kiru\n\nきる","Verb, Godan verb, Transitive verb","to cut"],["273","着る","kiru\n\nきる","Verb, Ichidan verb, Transitive verb","to wear"],["274","喫茶店","kissaten\n\nきっさてん","Noun","coffee shop; tearoom; cafe"],["275","北","kita\n\nきた","Noun","north"],["276","汚い","kitanai\n\nきたない","Adjective, い-adjective","dirty"],["277","切手","kitte\n\nきって","Noun","stamp (postage)"],["278","こっち","kocchi","Noun","this person or way"],["279","こちら","kochira","Noun","this way; this direction​"],["280","子供","kodomo\n\nこども","Noun","child"],["281","声","koe\n\nこえ","Noun","voice"],["282","ここ","koko","Pronoun","here; this place"],["283","九日","kokonoka\n\nここのか","Noun","ninth day of the month / 9 days"],["284","九つ","kokonotsu\n\nここのつ","Noun, Numeric","nine; 9"],["285","困る","komaru\n\nこまる","Verb, Godan verb, Intransitive verb","to be troubled"],["286","今晩","konban\n\nこんばん","Noun","tonight; this evening"],["287","今月","kongetsu\n\nこんげつ","Noun","this month"],["288","こんな","konna","Pre-noun adjectival","such; like this​"],["289","この","kono","Pre-noun adjectival","this​"],["290","今週","konshuu\n\nこんしゅう","Noun","this week"],["291","コーヒー","koohii","Noun, Katakana","Coffee"],["292","コート","kooto","Noun, Katakana","coat"],["293","コピー","kopii","Noun, Suru verb, Katakana","copy; photocopy"],["294","コップ","koppu","Noun, Katakana","glass (drinking vessel); tumbler​"],["295","これ","kore","Pronoun","this"],["296","答える","kotaeru\n\nこたえる","Verb, Ichidan verb, Intransitive verb","to answer"],["297","言葉","kotoba\n\nことば","Noun","word; words"],["298","今年","kotoshi\n\nことし","Noun","this year"],["299","交番","kouban\n\nこうばん","Noun","police box"],["300","紅茶","koucha\n\nこうちゃ","Noun","black tea"],["301","公園","kouen\n\nこうえん","Noun","park"],["302","交差点","kousaten\n\nこうさてん","Noun","intersection"],["303","口","kuchi\n\nくち","Noun","mouth, opening"],["304","果物","kudamono\n\nくだもの","Noun","fruit"],["305","下さい","kudasai\n\nください","Expression","please"],["306","曇り","kumori\n\nくもり","Noun","cloudiness; cloudy weather"],["307","曇る","kumoru\n\nくもる","Verb, Godan verb, Intransitive verb","to become cloudy, to become dim"],["308","国","kuni\n\nくに","Noun","country; state; region"],["309","暗い","kurai\n\nくらい","Adjective, い-adjective","dark; gloomy; murky"],["310","クラス","kurasu","Noun, Katakana","class"],["311","黒","kuro\n\nくろ","Noun","black"],["312","黒い","kuroi\n\nくろい","い-adjective","black"],["313","来る","kuru\n\nくる","Verb, Intransitive verb","to come"],["314","車","kuruma\n\nくるま","Noun","car; automobile; vehicle"],["315","薬","kusuri\n\nくすり","Noun","medicine"],["316","靴","kutsu\n\nくつ","Noun","shoes"],["317","靴下","kutsushita\n\nくつした","Noun","socks"],["318","去年","kyonen\n\nきょねん","Noun","last year"],["319","今日","kyou\n\nきょう","Noun, Temporal noun","today; this day"],["320","兄弟","kyoudai\n\nきょうだい","Noun","siblings; brothers and sisters​; mate"],["321","教室","kyoushitsu\n\nきょうしつ","Noun","classroom"],["322","九","kyuu\n\nきゅう","Noun, Numeric","nine; 9"],["323","マッチ","macchi","Noun, Katakana","match"],["324","町","machi\n\nまち","Noun","town; block; neighborhood"],["325","窓","mado\n\nまど","Noun","window"],["326","前","mae\n\nまえ","Noun, Suffix","previous; before; in front; ago"],["327","曲がる","magaru\n\nまがる","Verb, Godan verb, Intransitive verb","to turn, to bend"],["328","毎朝","maiasa\n\nまいあさ","Noun","every morning"],["329","毎晩","maiban\n\nまいばん","Noun","every night"],["330","毎日","mainichi\n\nまいにち","Noun","every day"],["331","毎週","maishuu\n\nまいしゅう","Noun","every week"],["332","毎年","maitoshi / mainen\n\nまいとし / まいねん","Noun","every year; yearly; annually"],["333","毎月","maitsuki\n\nまいつき","Noun","every month; monthly"],["334","万","man\n\nまん","Noun, Numeric","10,000; ten thousand"],["335","万年筆","mannenhitsu\n\nまんねんひつ","Noun","fountain pen"],["336","丸い","marui\n\nまるい","Adjective, い-adjective","round,circular"],["337","真っ直ぐ","massugu\n\nまっすぐ","Noun, Adjective, な-adjective, Adverb","straight ahead,direct"],["338","待つ","matsu\n\nまつ","Verb, Godan verb, Intransitive verb, Transitive verb","to wait​"],["339","不味い","mazui\n\nまずい","Adjective, い-adjective","unpleasant"],["340","目","me\n\nめ","Noun","eye"],["341","メートル","meetoru","Noun, Katakana","metre; meter"],["342","眼鏡","megane\n\nめがね","Noun","glasses"],["343","道","michi\n\nみち","Noun","road; street"],["344","緑","midori\n\nみどり","Noun","green"],["345","磨く","migaku\n\nみがく","Verb, Godan verb, Intransitive verb","to polish; to shine; to brush (e.g. teeth)"],["346","右","migi\n\nみぎ","Noun","right; right hand side"],["347","短い","mijikai\n\nみじかい","Adjective, い-adjective","short"],["348","三日","mikka\n\nみっか","Noun","the third day of the month / 3 days"],["349","耳","mimi\n\nみみ","Noun","ear; hearing"],["350","南","minami\n\nみなみ","Noun","south"],["351","皆さん","minasan\n\nみなさん","Noun","everyone"],["352","みんな","minna","Noun, Adverb","all; everyone; everybody"],["353","見る","miru\n\nみる","Verb, Ichidan verb, Transitive verb","to see; to look; to watch; to view; to observe"],["354","店","mise\n\nみせ","Noun","store; shop; establishment; restaurant"],["355","見せる","miseru\n\nみせる","Verb, Ichidan verb, Transitive verb","to show; to display"],["356","三つ","mitsu\n\nみつ","Noun, Numeric","three; 3"],["357","水","mizu\n\nみず","Noun","water; fluid; liquid​"],["358","木曜日","mokuyoubi\n\nもくようび","Noun","Thursday"],["359","門","mon\n\nもん","Noun","gate"],["360","問題","mondai\n\nもんだい","Noun","problem; question (e.g. on a test)"],["361","物","mono\n\nもの","Noun","thing"],["362","持つ","motsu\n\nもつ","Verb, Godan verb","to hold"],["363","もっと","motto","Adverb","more; longer; further"],["364","もう一度","mouichido\n\nもういちど","Expression","once more; again"],["365","六日","muika\n\nむいか","Noun","sixth day of the month / 6 days"],["366","向こう","mukou\n\nむこう","Noun","over there"],["367","村","mura\n\nむら","Noun","village"],["368","六つ","muttsu\n\nむっつ","Noun, Numeric","six; 6"],["369","難しい","muzukashii\n\nむずかしい","Adjective, い-adjective","difficult"],["370","長い","nagai\n\nながい","Adjective, い-adjective","long (distance)​; long (time); lengthy."],["371","ナイフ","naifu","Noun, Katakana","knife"],["372","中","naka\n\nなか","Noun","inside; in; within; center"],["373","鳴く","naku\n\nなく","Verb, Godan verb, Intransitive verb","animal noise. to chirp"],["374","無くす","nakusu\n\nなくす","Verb, Godan verb, Transitive verb","to lose (something)"],["375","名前","namae\n\nなまえ","Noun","name; full name; given name"],["376","七つ","nanatsu\n\nななつ","Noun, Numeric","seven; 7"],["377","何","nani\n\nなに","Noun, Pronoun, Prefix","what"],["378","七日","nanoka\n\nなのか","Noun","seventh day of the month / 7 days"],["379","並べる","naraberu\n\nならべる","Verb, Ichidan verb, Transitive verb","to line up,to set up"],["380","並ぶ","narabu\n\nならぶ","Verb, Godan verb, Intransitive verb","to line up,to stand in a line"],["381","習う","narau\n\nならう","Verb, Godan verb, Transitive verb","to be taught; to learn (from a teacher)"],["382","夏","natsu\n\nなつ","Noun","summer"],["383","夏休み","natsuyasumi\n\nなつやすみ","Noun","summer vacation; summer holiday"],["384","何故","naze\n\nなぜ","Adverb","why; how"],["385","猫","neko\n\nねこ","Noun","cat"],["386","ネクタイ","nekutai","Noun, Katakana","tie; necktie"],["387","寝る","neru\n\nねる","Verb, Ichidan verb, Intransitive verb","to sleep; to go to bed; to lie down"],["388","二","ni\n\nに","Noun, Numeric","two; 2"],["389","日曜日","nichiyoubi\n\nにちようび","Noun","Sunday"],["390","賑やか","nigiyaka\n\nにぎやか","Adjective, な-adjective","bustling,busy"],["391","日記","nikki\n\nにっき","Noun","diary; journal"],["392","肉","niku\n\nにく","Noun","meat"],["393","荷物","nimotsu\n\nにもつ","Noun","luggage; baggage"],["394","西","nishi\n\nにし","Noun","west"],["395","庭","niwa\n\nにわ","Noun","garden"],["396","登る","noboru\n\nのぼる","Verb, Godan verb, Intransitive verb","to climb"],["397","飲み物","nomimono\n\nのみもの","Noun","drink; beverage"],["398","飲む","nomu\n\nのむ","Verb, Godan verb, Transitive verb","to drink"],["399","ノート","nooto","Noun, Katakana","notebook"],["400","乗る","noru\n\nのる","Verb, Godan verb, Intransitive verb","to get on (train, plane, bus, ship, etc.)"],["401","脱ぐ","nugu\n\nぬぐ","Verb, Godan verb, Transitive verb","to take off clothes"],["402","温い","nurui\n\nぬるい","Adjective, い-adjective","luke warm"],["403","ニュース","nyuusu","Noun, Katakana","news"],["404","おばあさん","obaasan","Noun","grandmother"],["405","伯母さん","obasan\n\nおばさん","Noun","aunt; old lady"],["406","お弁当","obentou\n\nおべんとう","Noun","lunch box; Japanese box lunch"],["407","覚える","oboeru\n\nおぼえる","Verb, Ichidan verb, Transitive verb","to remember"],["408","お茶","ocha\n\nおちゃ","Noun","tea"],["409","お風呂","ofuro\n\nおふろ","Noun","bath"],["410","美味しい","oishii\n\nおいしい","い-adjective","delicious"],["411","伯父さん","ojisan\n\nおじさん","Noun","uncle; old man; mister"],["412","お母さん","okaasan\n\nおかあさん","Noun","mother; mom; mum; ma"],["413","お金","okane\n\nおかね","Noun","money"],["414","お菓子","okashi\n\nおかし","Noun","confections; sweets; candy"],["415","起きる","okiru\n\nおきる","Verb, Ichidan verb, Intransitive verb","to get up; to wake up"],["416","置く","oku\n\nおく","Verb, Godan verb, Transitive verb","to put; to place​"],["417","奥さん","okusan\n\nおくさん","Noun","wife; your wife; his wife"],["418","お巡りさん","omawari san\n\nおまわりさん","Noun","police officer (friendly term for policeman)"],["419","重い","omoi\n\nおもい","い-adjective","heavy"],["420","面白い","omoshiroi\n\nおもしろい","Adjective, い-adjective","interesting"],["421","同じ","onaji\n\nおなじ","Noun, Adverb","same"],["422","お腹","onaka\n\nおなか","Noun","stomach"],["423","お姉さん","oneesan\n\nおねえさん","Noun","elder sister; young lady; miss; ma'am"],["424","音楽","ongaku\n\nおんがく","Noun","music"],["425","お兄さん","oniisan\n\nおにいさん","Noun","older brother; elder brother; young man; buddy"],["426","女","onna\n\nおんな","Noun, Prefix","woman; female sex"],["427","女の子","onnanoko\n\nおんなのこ","Noun, Expression","girl; daughter; young women"],["428","多い","ooi\n\nおおい","Adjective, い-adjective","many; numerous; a lot; large quantity; frequent"],["429","大きい","ookii\n\nおおきい","Adjective, い-adjective","big; large; great; important"],["430","大きな","ookina\n\nおおきな","Pre-noun adjectival","big; large; great​"],["431","大勢","oozei\n\nおおぜい","Noun","crowd of people; great number of people"],["432","降りる","oriru\n\nおりる","Verb, Ichidan verb, Intransitive verb","to get off"],["433","お酒","osake\n\nおさけ","Noun","alcohol"],["434","お皿","osara\n\nおさら","Noun","plate, dish"],["435","教える","oshieru\n\nおしえる","Verb, Ichidan verb, Transitive verb","to teach"],["436","遅い","osoi\n\nおそい","Adjective, い-adjective","slow; time-consuming; late"],["437","押す","osu\n\nおす","Verb, Godan verb, Transitive verb","to push; to press​"],["438","お手洗い","otearai\n\nおてあらい","Noun","toilet; restroom; lavatory; bathroom"],["439","男","otoko\n\nおとこ","Noun","man; male"],["440","男の子","otokonoko\n\nおとこのこ","Noun, Expression","boy; male child; baby boy"],["441","大人","otona\n\nおとな","Noun","adult"],["442","一昨日","ototoi\n\nおととい","Noun","day before yesterday"],["443","一昨年","ototoshi\n\nおととし","Noun","year before last"],["444","お父さん","otousan\n\nおとうさん","Noun","father; dad; papa; pa; pop; daddy"],["445","弟","otouto\n\nおとうと","Noun","younger brother"],["446","終る","owaru\n\nおわる","Verb, Godan verb, Intransitive verb","to finish; to end"],["447","泳ぐ","oyogu\n\nおよぐ","Verb, Godan verb, Intransitive verb","to swim"],["448","パーティー","paatii","Noun, Katakana","party"],["449","パン","pan","Noun, Katakana","bread"],["450","ページ","peeji","Noun, Katakana","page"],["451","ペン","pen","Noun, Katakana","pen"],["452","ペット","petto","Noun, Katakana","pet"],["453","ポケット","poketto","Noun","pocket"],["454","ポスト","posuto","Noun, Katakana","post"],["455","プール","puuru","Noun, Katakana","swimming pool"],["456","来月","raigetsu\n\nらいげつ","Noun","next month"],["457","来年","rainen\n\nらいねん","Noun","next year"],["458","来週","raishuu\n\nらいしゅう","Noun","next week"],["459","ラジオ","rajiio","Noun, Katakana","radio"],["460","零","rei\n\nれい","Noun","zero"],["461","冷蔵庫","reizouko\n\nれいぞうこ","Noun","refrigerator"],["462","レコード","rekoodo","Noun, Katakana","record"],["463","練習","renshuu\n\nれんしゅう","Noun, Verb, Suru verb","practice; practicing"],["464","レストラン","resutoran","Noun, Katakana","restaurant"],["465","立派","rippa\n\nりっぱ","Adjective, な-adjective","splendid"],["466","六","roku\n\nろく","Noun, Numeric","six; 6"],["467","廊下","rouka\n\nろうか","Noun","corridor; hallway; passageway"],["468","旅行","ryokou\n\nりょこう","Noun, Verb, Suru verb","travel; trip; journey; excursion; tour"],["469","料理","ryouri\n\nりょうり","Noun, Suru verb","cuisine"],["470","両親","ryoushin\n\nりょうしん","Noun","parents; both parents"],["471","留学生","ryuugakusei\n\nりゅうがくせい","Noun","overseas student; exchange student"],["472","さあ","saa","Conjunction","well…"],["473","財布","saifu\n\nさいふ","Noun","purse; wallet"],["474","魚","sakana\n\nさかな","Noun","fish"],["475","先","saki\n\nさき","Noun, Prefix, Suffix","previous; prior; first; earlier"],["476","咲く","saku\n\nさく","Verb, Godan verb, Intransitive verb","to bloom"],["477","作文","sakubun\n\nさくぶん","Noun","writing; composition"],["478","寒い","samui\n\nさむい","Adjective, い-adjective","cold"],["479","三","san\n\nさん","Noun, Numeric","three; 3"],["480","散歩","sanpo\n\nさんぽ","Noun, Verb, Suru verb","walk; stroll"],["481","再来年","sarainen\n\nさらいねん","Noun","year after next"],["482","差す","sasu\n\nさす","Verb, Godan verb, Transitive verb","to stretch out hands, to raise an umbrella"],["483","砂糖","satou\n\nさとう","Noun","sugar"],["484","背","se\n\nせ","Noun","height; stature; back; spine"],["485","背広","sebiro\n\nせびろ","Noun","business suit"],["486","セーター","seetaa","Noun, Katakana","sweater; jumper"],["487","生徒","seito\n\nせいと","Noun","pupil; student"],["488","石鹼","sekken\n\nせっけん","Noun","soap"],["489","狭い","semai\n\nせまい","い-adjective","narrow"],["490","千","sen\n\nせん","Noun, Numeric","1,000; thousand"],["491","先月","sengetsu\n\nせんげつ","Noun","last month"],["492","先生","sensei\n\nせんせい","Noun, Suffix","teacher; instructor; master"],["493","先週","senshuu\n\nせんしゅう","Noun","last week"],["494","洗濯","sentaku\n\nせんたく","Noun, Suru verb","washing; laundry"],["495","写真","shashin\n\nしゃしん","Noun","photograph; photo"],["496","シャツ","shatsu","Noun, Katakana","shirt"],["497","シャワー","shawaa","Noun, Katakana","shower"],["498","四","shi\n\nし","Noun, Numeric","four; 4"],["499","七","shichi\n\nしち","Noun, Numeric","seven; 7"],["500","仕事","shigoto\n\nしごと","Noun, Verb, Suru verb","work; job; business"],["501","閉まる","shimaru\n\nしまる","Verb, Godan verb, Intransitive verb","to close, to be closed"],["502","締める","shimeru\n\nしめる","Verb, Ichidan verb, Transitive verb","to tie; to fasten; to tighten​"],["503","閉める","shimeru\n\nしめる","Verb, Ichidan verb, Transitive verb","to close; to shut"],["504","新聞","shinbun\n\nしんぶん","Noun","newspaper"],["505","死ぬ","shinu\n\nしぬ","Verb, Godan verb, Intransitive verb","to die"],["506","塩","shio\n\nしお","Noun","salt"],["507","白","shiro\n\nしろ","Noun","white; innocence; innocent person"],["508","白い","shiroi\n\nしろい","Adjective, い-adjective","white"],["509","知る","shiru\n\nしる","Verb, Godan verb, Transitive verb","to know"],["510","下","shita\n\nした","Noun","below; down; under; bottom"],["511","質問","shitsumon\n\nしつもん","Noun, Verb, Suru verb","question; inquiry"],["512","静か","shizuka\n\nしずか","Adjective, な-adjective","quiet"],["513","食堂","shokudou\n\nしょくどう","Noun","cafeteria; dining room"],["514","醬油","shouyu\n\nしょうゆ","Noun","soy sauce"],["515","宿題","shukudai\n\nしゅくだい","Noun","homework; assignment; pending issue"],["516","そば","soba","Noun","near; beside"],["517","そっち","socchi","Pronoun","that way; ​over there"],["518","そちら","sochira","Pronoun","that way (distant from speaker, close to listener); you; your family"],["519","そこ","soko","Pronoun","that place​; there"],["520","その","sono","Pre-noun adjectival","that"],["521","空","sora\n\nそら","Noun","sky; the air"],["522","それ","sore","Pronoun","that"],["523","それでは","sore dewa","Expression","in that situation"],["524","外","soto\n\nそと","Noun","outside; exterior;"],["525","掃除","souji\n\nそうじ","Noun, Suru verb","to clean, to sweep"],["526","直ぐに","sugu ni\n\nすぐに","Adverb","immediately; right away; instantly​"],["527","水曜日","suiyoubi\n\nすいようび","Noun","Wednesday"],["528","スカート","sukaato","Noun, Katakana","skirt"],["529","好き","suki\n\nすき","Noun, Adjective, な-adjective","like"],["530","少し","sukoshi\n\nすこし","Noun, Adverb","a little (bit); small quantity; few; short distance"],["531","少ない","sukunai\n\nすくない","い-adjective","few; a little; scarce; insufficient; seldom"],["532","住む","sumu\n\nすむ","Verb, Godan verb, Intransitive verb","to live in; to reside; to inhabit; to dwell; to abide"],["533","スポーツ","supootsu","Noun, Katakana","sport; sports"],["534","スプーン","supuun","Noun, Katakana","spoon"],["535","スリッパ","surippa","Noun, Katakana","slipper; slippers"],["536","ストーブ","sutoobu","Noun, Katakana","heater; stove"],["537","吸う","suu\n\nすう","Verb, Godan verb, Transitive verb","to smoke, to suck"],["538","座る","suwaru\n\nすわる","Verb, Godan verb, Intransitive verb","to sit"],["539","涼しい","suzushii\n\nすずしい","Adjective, い-adjective","refreshing, cool"],["540","たばこ","tabako","Noun","tobacco; cigarette"],["541","食べ物","tabemono\n\nたべもの","Noun","food"],["542","食べる","taberu\n\nたべる","Verb, Ichidan verb, Intransitive verb","to eat"],["543","多分","tabun\n\nたぶん","Adjective, な-adjective, Adverb","perhaps; probably"],["544","大変","taihen\n\nたいへん","Noun, Adjective, な-adjective, Adverb","very; greatly; terribly; serious; difficult"],["545","大切","taisetsu\n\nたいせつ","Noun, Adjective, な-adjective","important; necessary; indispensable; beloved"],["546","大使館","taishikan\n\nたいしかん","Noun","embassy"],["547","高い","takai\n\nたかい","Adjective, い-adjective","high; tall; expensive; above average"],["548","沢山","takusan\n\nたくさん","Noun, Adverbial Noun, Adjective, な-adjective","many"],["549","タクシー","takushii","Noun, Katakana","taxi"],["550","卵","tamago\n\nたまご","Noun","eggs; egg"],["551","誕生日","tanjoubi\n\nたんじょうび","Noun","birthday"],["552","頼む","tanomu\n\nたのむ","Verb, Godan verb, Transitive verb","to ask"],["553","楽しい","tanoshii\n\nたのしい","Adjective, い-adjective","enjoyable; fun"],["554","縦","tate\n\nたて","Noun","length,height"],["555","建物","tatemono\n\nたてもの","Noun","building"],["556","立つ","tatsu\n\nたつ","Verb, Godan verb, Intransitive verb","to stand; to stand up​"],["557","手","te\n\nて","Noun","hand; arm"],["558","テーブル","teeburu","Noun, Katakana","table"],["559","テープ","teepu","Noun, Katakana","tape"],["560","テープレコーダー","teepu rekoodaa","Noun, Katakana","tape recorder"],["561","手紙","tegami\n\nてがみ","Noun","Letter (message)​"],["562","天気","tenki\n\nてんき","Noun","weather; the elements"],["563","テレビ","terebi","Noun, Katakana","television; TV​"],["564","テスト","tesuto","Noun, Suru verb, Katakana","examination; quiz; test"],["565","戸","to\n\nと","Noun","Japanese style door"],["566","飛ぶ","tobu\n\nとぶ","Verb, Godan verb, Intransitive verb","to fly; to hop"],["567","トイレ","toire","Noun, Katakana","toilet"],["568","時計","tokei\n\nとけい","Noun","watch; clock; timepiece"],["569","時","toki\n\nとき","Noun","time; moment; occasion; chance"],["570","時々","tokidoki\n\nときどき","Noun, Adverb","sometimes; at times"],["571","所","tokoro\n\nところ","Noun, Suffix","place"],["572","止まる","tomaru\n\nとまる","Verb, Godan verb, Intransitive verb","to stop; to come to a halt"],["573","友達","tomodachi\n\nともだち","Noun","friend; companion"],["574","隣","tonari\n\nとなり","Noun","next door to"],["575","遠い","tooi\n\nとおい","Adjective, い-adjective","far"],["576","十日","tooka\n\nとおか","Noun","tenth day of the month / 10 days"],["577","鳥","tori\n\nとり","Noun","bird"],["578","鶏肉","toriniku\n\nとりにく","Noun","chicken meat"],["579","撮る","toru\n\nとる","Verb, Godan verb, Transitive verb","to take a photo or record a film"],["580","取る","toru\n\nとる","Verb, Godan verb, Transitive verb","to take; to pick up; to harvest; to earn; to win; to choose"],["581","年","toshi\n\nとし","Noun","year; age"],["582","図書館","toshokan\n\nとしょかん","Noun","library"],["583","次","tsugi\n\nつぎ","Noun","next"],["584","一日","tsuitachi\n\nついたち","Noun","first day of the month"],["585","疲れる","tsukareru\n\nつかれる","Verb, Ichidan verb, Intransitive verb","to get tired"],["586","使う","tsukau\n\nつかう","Verb, Godan verb, Transitive verb","to use"],["587","つける","tsukeru","Verb, Ichidan verb, Transitive verb","to turn on"],["588","着く","tsuku\n\nつく","Verb, Godan verb, Intransitive verb","to arrive at"],["589","机","tsukue\n\nつくえ","Noun","desk"],["590","作る","tsukuru\n\nつくる","Verb, Godan verb, Transitive verb","to make"],["591","詰まらない","tsumaranai\n\nつまらない","Adjective, い-adjective","boring"],["592","冷たい","tsumetai\n\nつめたい","い-adjective","cold to the touch"],["593","勤める","tsutomeru\n\nつとめる","Verb, Ichidan verb, Transitive verb","to work for someone"],["594","強い","tsuyoi\n\nつよい","Adjective, い-adjective","powerful"],["595","上","ue\n\nうえ","Noun, Suffix","above; up; over; top; surface"],["596","生まれる","umareru\n\nうまれる","Verb, Ichidan verb, Intransitive verb","to be born"],["597","海","umi\n\nうみ","Noun","sea"],["598","売る","uru\n\nうる","Verb, Godan verb, Transitive verb","to sell"],["599","煩い","urusai\n\nうるさい","い-adjective","noisy, annoying"],["600","後ろ","ushiro\n\nうしろ","Noun","back; behind; rear"],["601","薄い","usui\n\nうすい","Adjective, い-adjective","thin; weak"],["602","歌","uta\n\nうた","Noun","song"],["603","歌う","utau\n\nうたう","Verb, Godan verb, Intransitive verb, Transitive verb","to sing"],["604","上着","uwagi\n\nうわぎ","Noun","coat; tunic; jacket; outer garment"],["605","ワイシャツ","wai shatsu","Noun, Katakana, Wasei","shirt"],["606","若い","wakai\n\nわかい","Adjective, い-adjective","young"],["607","分かる","wakaru\n\nわかる","Verb, Godan verb, Intransitive verb","to understand; to comprehend; to grasp; to see; to get; to follow"],["608","悪い","warui\n\nわるい","Adjective, い-adjective","bad; poor; undesirable"],["609","忘れる","wasureru\n\nわすれる","Verb, Ichidan verb, Transitive verb","to forget"],["610","渡る","wataru\n\nわたる","Verb, Godan verb, Intransitive verb","to go across"],["611","私","watashi\n\nわたし","Pronoun","I; myself"],["612","渡す","watasu\n\nわたす","Verb, Godan verb, Transitive verb","to hand over"],["613","山","yama\n\nやま","Noun","mountain; hill"],["614","八百屋","yaoya\n\nやおや","Noun","greengrocer; fruit and vegetable shop; versatile"],["615","やる","yaru","Verb, Godan verb, Transitive verb","to do"],["616","野菜","yasai\n\nやさい","Noun","vegetable"],["617","易しい","yasashii\n\nやさしい","Adjective, い-adjective","easy, simple"],["618","安い","yasui\n\nやすい","Adjective, い-adjective","cheap; inexpensive"],["619","休み","yasumi\n\nやすみ","Noun","rest; vacation; holiday"],["620","休む","yasumu\n\nやすむ","Verb, Godan verb, Intransitive verb","to be absent; to take a day off; to rest"],["621","八つ","yattsu\n\nやっつ","Noun, Numeric","eight: 8"],["622","呼ぶ","yobu\n\nよぶ","Verb, Godan verb, Transitive verb","to call out, to invite"],["623","良い","yoi/ii\n\nよい/いい","Adjective, い-adjective","good"],["624","四日","yokka\n\nよっか","Noun","fourth day of the month / 4 days"],["625","横","yoko\n\nよこ","Noun","beside,side,width"],["626","よく","yoku","Adverb","often, well"],["627","読む","yomu\n\nよむ","Verb, Godan verb, Transitive verb","to read; to guess; to predict; to read (someone's thoughts)"],["628","夜","yoru\n\nよる","Noun","evening; night"],["629","四つ","yotsu\n\nよつ","Noun, Numeric","four; 4"],["630","洋服","youfuku\n\nようふく","Noun","western clothes"],["631","八日","youka\n\nようか","Noun","eighth day of the month / 8 days"],["632","弱い","yowai\n\nよわい","Adjective, い-adjective","weak"],["633","雪","yuki\n\nゆき","Noun","snow"],["634","ゆっくり","yukkuri","Adverb","slowly"],["635","昨夜","yuube\n\nゆうべ","Noun","last night"],["636","郵便局","yuubinkyoku\n\nゆうびんきょく","Noun","post office"],["637","夕方","yuugata\n\nゆうがた","Noun","evening; dusk"],["638","夕飯","yuuhan\n\nゆうはん","Noun","evening meal"],["639","有名","yuumei\n\nゆうめい","Noun, Adjective, な-adjective","famous"],["640","雑誌","zasshi\n\nざっし","Noun","magazine"],["641","全部","zenbu\n\nぜんぶ","Noun, Adverbial Noun","all"],["642","ゼロ","zero","Noun","zero"],["643","ズボン","zubon","Noun, Katakana","trousers; pants"]];

function getRandomWordData() {
	var random_row = random(jlptn5words);
	random_row[5] = random_row[4].split(' ').map(x => x.split('').filter(x => /\w/.test(x)).join('').toLowerCase());
	return random_row;
}




function startVocabularyGame(channel) {
	let w = () => `${channel.vgd[2].split('\n\n')[0]} (${channel.vgd[1]})`
	channel.vgd ||= client.on("messageCreate", async message => {
		if (message.channel.id == channel.id && message.author.id != client.user.id)
			if (message.content.toLowerCase().split(' ').some(word => channel.vgd[5].includes(word)) || message.content.toLowerCase() == "idk") {
				await message.reply("__*" + channel.vgd[3] + ':* ' + channel.vgd[4] + "__");
				channel.vgd = getRandomWordData();
				message.channel.send(`**Next:** ${w()}`);
			}
	}) && getRandomWordData();

	return w();
}







commands.push({
	name: "vocabularygame",
	description: "Start JLPT N5 vocabulary game",
	exec: i => i.reply(startVocabularyGame(i.channel))
})